## 
# Global Functions
# Author - David Fidge
# 
# Version 1.3
# - fixed a bug in getNext functions that would have caused earthquake damage to be applied to a far away square.
# Version 1.2
# Initial release
#


from CvPythonExtensions import *
from RandomEventsConstants import *

import CvUtil
import PyHelpers

PyPlayer = PyHelpers.PyPlayer
PyCity = PyHelpers.PyCity

gc = CyGlobalContext()
CyMapInstance = gc.getMap()


class GlobalFunctions:

	
	def getHuman(self):
		"get the human player"
		for iPlayerLoop in range(gc.getMAX_PLAYERS()):
			if (PyPlayer(iPlayerLoop).getPlayer().isHuman()):
				#CvUtil.pyPrint("gethuman returning " + str(iPlayerLoop))
				return iPlayerLoop

		#no human
		#CvUtil.pyPrint("gethuman returning -1")
		return -1

	def getTechByName(self, techstring):

		"compare textual description of each tech in turn to see if its techstring then return the ID"
		"NOTE NO LONGER IN USE, I FOUND OUT ABOUT TechTypes"

		for i in range(gc.getNumTechInfos()):

			#CvUtil.pyPrint("testing tech " + gc.getTechInfo(i).getDescription().upper())
			
			if(gc.getTechInfo(i).getDescription().upper() == techstring.upper()):
				#CvUtil.pyPrint("gettech returning " + str(i) + " which is " + gc.getTechInfo(i).getDescription().upper())
				return i
		
		#couldnt find tech
		#CvUtil.pyPrint("gettech returning -1")
		return -1

	##UNTESTED
	def calcDistance(self, srcX, srcY, destX, destY):
		"calculate the distance between 2 points on the map"

		mapX = CyMapInstance.getGridWidth()
		mapY = CyMapInstance.getGridHeight()

		newsrcX = srcX
		newsrcY = srcY
		newdestX = destX
		newdestY = destY

		#first we must account for map wrapping.

		if (CyMapInstance.isWrapX() == true):
			if (srcX < destX):
				if (destX - srcX > destX - (srcX + mapX)):
					newsrcX = srcX + mapX
			elif (destX < srcX):
				if (srcX - destX > srcX - (destX + mapX)):
					newdestX = destX + mapX

		if (CyMapInstance.isWrapY() == true):
			if (srcY < destY):
				if (destY - srcY > destY - (srcY + mapY)):
					newsrcY = srcY + mapY
			elif (destY < srcY):
				if (srcY - destY > srcY - (destY + mapY)):
					newdestY = destY + mapY
		
		#now calculate distance x = sqrroot(y^2 + z^2)
		#truncate decimal points by converting to int

		distance = int(math.sqrt(pow(destX-srcX,2) + pow(destY-srcY,2)))
		
		#CvUtil.pyPrint("distance between plots is " + str(distance))
		
		return distance

	##UNTESTED
	def getWithinDistance(self, dist, srcX, srcY):
		"returns a list of cities within the given distance from the given point"

		destX = 0
		destY = 0
		distance = 0
		playerList = []
		cityList = []
		distanceList = []

		for iPlayerLoop in range(gc.getMAX_PLAYERS()):
			
			apCityList = PyPlayer(iPlayerLoop).getCityList()
			
			for pCity in apCityList:

				destX = pCity.city.plot().getX()
				destY = pCity.city.plot().getY()

				distance = self.calcDistance(srcX, srcY, destX, destY)

				if (distance <= dist):

					cityList.append(pCity)
					playerList.append(iPlayerLoop)
					distanceList.append(distance)
		
		return cityList, playerList, distanceList



	def getNextX(self, x, amount):
		"gets the next valid map square when moving along x axis"
		"input - x = current x pos, amount = how far to move from x (can be +ve or -ve)"

		diff = 0
		toadd = 0
		tempx = x + amount
		if (amount > 0):

			if (tempx >= CyMapInstance.getGridWidth()):  #account for map wrap
							
				if (CyMapInstance.isWrapX() == true):   #does map wrap
					diff = CyMapInstance.getGridWidth() - x
					toadd = amount - diff
					return 0 + toadd
				else:
					#any negative value is not a valid map value so use it to signify error
					return -1
			else:
				return tempx

		elif (amount < 0):
			if (tempx < 0):  #account for map wrap
							
				if (CyMapInstance.isWrapX() == true):   #does map wrap
					diff = -x 
					toadd = amount - diff
					return CyMapInstance.getGridWidth() + toadd
				else:
					return -1
			else:
				return tempx

		else:
			return x 


	def getNextY(self, y, amount):
		"gets the next valid map square when moving along y axis"
		"input - y = current y pos, amount = how far to move from y (can be +ve or -ve)"
		diff = 0
		toadd = 0
		tempy = y + amount
		if (amount > 0):

			if (tempy >= CyMapInstance.getGridHeight()):  #account for map wrap
							
				if (CyMapInstance.isWrapY() == true):   #does map wrap
					diff = CyMapInstance.getGridHeight() - y
					toadd = amount - diff
					return 0 + toadd
				else:
					#any negative value is not a valid map value so use it to signify error
					return -1
			else:
				return tempy

		elif (amount < 0):
			if (tempy < 0):  #account for map wrap
							
				if (CyMapInstance.isWrapY() == true):   #does map wrap
					diff = -y 
					toadd = amount - diff
					return CyMapInstance.getGridHeight() + toadd
				else:
					return -1
			else:
				return tempy

		else:
			return y 

	def getAllCities(self):

		cityList = []

		for iPlayerLoop in range(gc.getMAX_PLAYERS()):
			
			apCityList = PyPlayer(iPlayerLoop).getCityList()
			
			for pCity in apCityList:

				cityList.append(pCity)


 
					