## 
# Landmark Manager
# Author - David Fidge
# 
# Version 1.4
# - initial release
#
#This class allows the programmer to write a landmark to a plot without losing the existing landmark.

from CvPythonExtensions import *
from RandomEventsConstants import *


import CvUtil

gc = CyGlobalContext()
CyMapInstance = gc.getMap()


class LandmarkManager:


	def removeLandmark(self, plot):
		"removes the entire landmark text"
		CyEngine().removeLandmark(plot)

	def addLandmark(self, plot, text):
		"adds a landmark to a plot, replacing the existing one"
		CyEngine().addLandmark(plot, str(text))

	def addLandmarkText(self, plot, text):
		existingtext = self.getLandmarkText(plot)

		if (existingtext == ""):
			self.addLandmark(plot, text)

		else:
			self.removeLandmark(plot)

			CvUtil.pyPrint("Adding landmark: " + str(existingtext + "\n" + text))
			self.addLandmark(plot, str(existingtext + "\n" + text))


	def removeLandmarkText(self, plot, text):
		"removes text from a landmark."
		"input:  text = if text is found as a string or substring in any of the splitted strings, then the entire splitted string will be removed."

		existingtext = self.getLandmarkText(plot)

		CvUtil.pyPrint("existing text = " + existingtext)

		findstring = existingtext.find(text)

		newstring = ""

		if (findstring != -1):

			"found substring text in landmark's text, remove the landmark"
			
			self.removeLandmark(plot)

			"split the landmark strings"

			splittedstrings = existingtext.split("\n")

			CvUtil.pyPrint("splitted strings is: " + str(splittedstrings))

			"if there is more than one string in the landmark then we want to keep the other strings, so a landmark with the other strings will need to be created"
			
			if (len(splittedstrings) > 1):
			
				for x in range (len(splittedstrings)):
					"if the splitted string is not the string to be removed then add the string to the landmark text"
					if (splittedstrings[x].find(text) == -1):
						newstring = newstring + splittedstrings[x] + "\n"

				"strip off the last newline"

				newstring.rstrip('\n')

				"add the landmark to the plot"

				self.addLandmark(plot, newstring)
					

	def getLandmarkText(self, plot):
		"gets landmark text in current plot, if no landmark found, return empty string"

		#CvUtil.pyPrint("TEST FOR PLOT " + str(plot) + " AT " + str(plot.getX()) + "," + str(plot.getY()))
		
		for x in range(CyEngine().getNumSigns()):

			#CvUtil.pyPrint("searching plot " + str(CyEngine().getSignByIndex(x).getPlot()) + " with caption: " + CyEngine().getSignByIndex(x).getCaption() + "at " + str(CyEngine().getSignByIndex(x).getPlot().getX()) + "," + str(CyEngine().getSignByIndex(x).getPlot().getY()))

			#seems to be a bug in comparing getPlots because the reference are the same for every getPlot call in getSignByIndex(), could be a pointer to pointer in code and its returning the pointer to pointer all the time??
			#anyway I still get the corret x, y and caption so compare plots by x and y
		
			if (CyEngine().getSignByIndex(x).getPlot().getX() == plot.getX() and CyEngine().getSignByIndex(x).getPlot().getY() == plot.getY()):
		
				return CyEngine().getSignByIndex(x).getCaption()

		return ""

	def getLandmark(self, plot):
		"equivalent to getLandmarkText"

		return self.getLandmarkText(plot)


					