## 
# Earthquake Event
# Author - David Fidge
# 
# Version 1.2
# - removed some constants
# Version 1.1
# - initial release
#

from CvPythonExtensions import *
from RandomEventsConstants import *
from CvRandomEventsScreenEnums import *
import CvRandomEventsScreenUtils
import GlobalFunctions

import RandomEventsIO
import RandomEventsScreen

import CvUtil
import PyHelpers

import random
import math

#import CvScreenEnums

PyPlayer = PyHelpers.PyPlayer
PyCity = PyHelpers.PyCity
GlobalFunctions = GlobalFunctions.GlobalFunctions()
gc = CyGlobalContext()
CyMapInstance = gc.getMap()


#the main class

class RandomEventEarthquake:
	"Earthquake Event Class"
	
	#class initialisation (called on creation of the class instance within CvScreensInterface)
	
	def __init__(self, io):


		self.earthquakePlot = 0
		self.earthquakeCity = 0
		self.severity = 0

		#era the player is in
		self.era = EARTHQUAKE_PRE_SEISMIC_ERA

		#set the io object - should always be done last as a reminder to not use the object yet 
		#as game may have not started

		#no IO in this event, but keep it here just in case I want to change that later
		self.IOinstance = io

	#initialise the mod.  This is called when loading a game or starting a new game

	def initialise(self):
		"earthquake severity is shown when scientific method is invented. "

		self.era = EARTHQUAKE_PRE_SEISMIC_ERA

		if (PyPlayer(GlobalFunctions.getHuman()).hasResearchedTech(gc.getInfoTypeForString("TECH_SCIENTIFIC_METHOD"))): 

			self.changeToModern()	

	def checkEra(self):
		"when event is called upon, this is done to check the era"

		if (PyPlayer(GlobalFunctions.getHuman()).hasResearchedTech(gc.getInfoTypeForString("TECH_SCIENTIFIC_METHOD"))): 

			self.changeToModern()	
		

	def changeToModern(self):

		"called during initialisation and also to change to modern era halfway through a game"

		self.era = EARTHQUAKE_POST_SEISMIC_ERA	


	def showScreen(self):
		"show the earthquake popup"

		#playerList = []
		#cityList = []
		#distanceList = []
		#damageList = []

		poploss = 0
		destroyedbuildingtext = ""
		damagedone = false

		#self.earthquakePlot = CyMapInstance.plot(5, 5)	#DEBUG

		#cityList, playerList, distanceList = getWithinDistance(3, self.earthquakePlot.getX(), self.earthquakePlot.getY())

		if (self.severity < EARTHQUAKE_SEVERITY1):
			#if centre is not a city 
			if (not self.earthquakePlot.isCity()):
	
				#chance of 1 plot improvement destroyed = severity%
				if (self.destroyOneRandom(EARTHQUAKE_MAXPOWER, self.severity, self.earthquakePlot)):
					CyInterface().addImmediateMessage("The earthquake destroyed plot improvements", "")
					damagedone = true


		elif (self.severity < EARTHQUAKE_SEVERITY2):
			#if centre is not a city 
			if (not self.earthquakePlot.isCity()):

				#test all improvements for improvement destroyed = severity%
				if (self.destroyRouteRandom(EARTHQUAKE_MAXPOWER, self.severity, self.earthquakePlot) or self.destroyImprovementRandom(EARTHQUAKE_MAXPOWER, self.severity, self.earthquakePlot)):
					CyInterface().addImmediateMessage("The earthquake destroyed plot improvements", "")
					damagedone = true

			#if centre is city then
			elif (self.earthquakePlot.isCity()):

				#chance of 1 building destroyed = severity% 
				destroyedbuildingtext = self.destroyOneBuildingRandom(EARTHQUAKE_MAXPOWER, self.severity, self.earthquakePlot.getPlotCity())
				if (destroyedbuildingtext != ""):
					CyInterface().addImmediateMessage(self.earthquakePlot.getPlotCity().getName() + " lost " + destroyedbuildingtext + " from the earthquake", "")
					damagedone = true

		elif (self.severity < EARTHQUAKE_SEVERITY3):
			#if centre is not a city 
			if (not self.earthquakePlot.isCity()):

				#destroy all improvements on land plot
				if (self.destroyAll(self.earthquakePlot)):
					CyInterface().addImmediateMessage("The earthquake destroyed plot improvements", "")
					damagedone = true

			#if centre is city then
			elif (self.earthquakePlot.isCity()):
				damagedone = true
				#destroy 1 building
				destroyedbuildingtext = self.destroyOneBuilding(self.earthquakePlot.getPlotCity())
				if (destroyedbuildingtext != ""):
					CyInterface().addImmediateMessage(self.earthquakePlot.getPlotCity().getName() + " lost " + destroyedbuildingtext + " from the earthquake", "")

				#populationloss = population / 10
				poploss = self.doPopulationLoss(self.earthquakePlot.getPlotCity(), 10)

				#write pop loss to quick messages
				if (poploss > 0):
					CyInterface().addImmediateMessage(self.earthquakePlot.getPlotCity().getName() + " lost " + str(poploss) + " population from the earthquake!", "")

		elif (self.severity <= EARTHQUAKE_MAXPOWER):

			
			#if centre is not a city 
			if (not self.earthquakePlot.isCity()):

				#destroy all improvements on land plot
				if (self.destroyAll(self.earthquakePlot)):
					#CyInterface().addImmediateMessage("The earthquake destroyed plot improvements", "")
					#havent put this message in because it is more than likely the "destroyed many plot improvements" will be shown
					damagedone = true

			#if centre is city then
			elif (self.earthquakePlot.isCity()):
				damagedone = true
				#destroy 1 building
				destroyedbuildingtext = self.destroyOneBuilding(self.earthquakePlot.getPlotCity())
				if (destroyedbuildingtext != ""):
					CyInterface().addImmediateMessage(self.earthquakePlot.getPlotCity().getName() + " lost " + destroyedbuildingtext + " from the earthquake", "")

				#for all buildings must roll, chance of destruction = 10%
				destroyedbuildingtext = self.destroyAllBuildingsRandom(EARTHQUAKE_MAXPOWER, 10, self.earthquakePlot.getPlotCity())
				if (destroyedbuildingtext != ""):
					CyInterface().addImmediateMessage(self.earthquakePlot.getPlotCity().getName() + " lost " + destroyedbuildingtext + " from the earthquake!!", "")

				#population loss = population / 3
				poploss = self.doPopulationLoss(self.earthquakePlot.getPlotCity(), 3)

				#write pop loss to quick messages
				if (poploss > 0):
					CyInterface().addImmediateMessage(self.earthquakePlot.getPlotCity().getName() + " lost " + str(poploss) + " population from the earthquake!!", "")
	

			manyplotimprovementflag = false
			#for all plots surrounding this plot
			for x in range (-1, 2):
				for y in range (-1, 2):
					if (not (x == 0 and y == 0)):
						#CvUtil.pyPrint("x = " + str(x) + ", y = " + str(y))

						tempx = GlobalFunctions.getNextX(self.earthquakePlot.getX(), x)
						tempy = GlobalFunctions.getNextY(self.earthquakePlot.getY(), y)

						if (tempx != -1 and tempy != -1):
											
        						seekplot = CyMapInstance.plot(tempx, tempy)
        
                        				#if plot is not a city then
        						if (not seekplot.isCity()):
        
                        					#destroy all improvements on plot
        							if (self.destroyAll(seekplot)):
									damagedone = true
									if (not manyplotimprovementflag):
										manyplotimprovementflag = true	
										CyInterface().addImmediateMessage("The earthquake destroyed many plot improvements!", "")
	        
                        				#if city plot 
        						elif (seekplot.isCity()):
								damagedone = true
                        					#population loss = population / 5
                                				poploss = self.doPopulationLoss(seekplot.getPlotCity(), 5)
        
                                				#write pop loss to quick messages
								if (poploss > 0):
	                                				CyInterface().addImmediateMessage(seekplot.getPlotCity().getName() + " lost " + str(poploss) + " population from the earthquake!", "")
                                	
                        					#destroy 1 building
                        					destroyedbuildingtext = self.destroyOneBuilding(seekplot.getPlotCity())
        							if (destroyedbuildingtext != ""):
        								CyInterface().addImmediateMessage(seekplot.getPlotCity().getName() + " lost " + destroyedbuildingtext + " from the earthquake", "")


		#put popup in the "screens that need to be shown queue"

		CvRandomEventsScreenUtils.randomEventsScreen.pushScreen(EARTHQUAKE_QUEUE_ID)

		#set variables in the popup screen so it can display certain information

		CvRandomEventsScreenUtils.randomEventsScreen.setEarthquakeScreenVars(self.earthquakePlot, self.severity, self.era, damagedone)

	def doPopulationLoss(self, city, divider):
		"does population loss for earthquake"

		currentPop = city.getPopulation()
		city.changePopulation(-(currentPop / divider))
		return (currentPop / divider)

	def destroyOneBuildingRandom(self, randommax, randomlimit, city):
		"destroys 1 building at random in the given city, returns name of building destroyed"

		text = ""
		randnum = random.randint(1, randommax)
		if (randnum <= randomlimit):
			return self.destroyOneBuilding(city)
		return text

	def destroyOneBuilding(self, city):
		"destroys 1 building in the given city if the city has any, returns name of building destroyed"

		list = []
		list = self.getBuildingList(city)
		text = ""

		if (len(list) > 0):
        		randnum = random.randint(0, len(list) - 1)
        		return self.destroyBuilding(city, list[randnum])
		return text
		
	def getBuildingList(self, city):
		"get a list of the non-wonder real buildings in a city"

		list = []

		#dont include wonders
		for b in range(gc.getInfoTypeForString("BUILDING_WALLS"), gc.getInfoTypeForString("BUILDING_TAOIST_SHRINE")): 
			#city has this building?
			if (city.hasBuilding(b)):
				#not a freebie from a wonder?
				if (not city.isFreeBuilding(b)):
					list.append(b)

		return list
		

	def destroyAllBuildingsRandom(self, randommax, randomlimit, city):
		"does a roll for all buildings in a city, if the building fails the roll it is destroyed"

		list = []
		list = self.getBuildingList(city)
		destroyedstring = ""
		text = ""
		
		#dont include wonders
		for b in list:
			randnum = random.randint(1, randommax)
			if (randnum <= randomlimit):
				text = self.destroyBuilding(city, b)
				if (destroyedstring == ""):
					destroyedstring += text
				else:
					destroyedstring = destroyedstring + ", " + text

		return destroyedstring

	def destroyBuilding(self, city, b):
		"removes a building from a city and returns its name"

		city.setHasRealBuilding(b, 0)
		return gc.getBuildingInfo(b).getText()
		
			
	def destroyOneRandom(self, randommax, randomlimit, plot):
		"random chance of destroying either a road/railroad or improvement"

		randnum = random.randint(1, randommax)
		if (randnum <= randomlimit):
			return self.destroyOne(plot)

		return false
			

	def destroyOne(self, plot):
		"destroys either a road/railroad or improvement at random"

		randnum = random.randint(1, 2)
		if (randnum == 1):
			return self.destroyRoute(plot)
		else:
			return self.destroyImprovement(plot)


	def destroyRouteRandom(self, randommax, randomlimit, plot):
		"destroys a route if roll fails"
		randnum = random.randint(1, randommax)
		if (randnum <= randomlimit):
			return self.destroyRoute(plot)

		return false

	def destroyImprovementRandom(self, randommax, randomlimit, plot):
		"destroys an improvement if roll fails"
		randnum = random.randint(1, randommax)
		if (randnum <= randomlimit):
			return self.destroyImprovement(plot)

		return false

	def destroyAll(self, plot):
		"destroys all routes and improvements" 

		if (self.destroyRoute(plot) or self.destroyImprovement(plot)):
			return true

		return false

		

	def destroyRoute(self, plot):
		"destroys rail/road"
		if (plot.getRouteType() != gc.getInfoTypeForString("NO_ROUTE")):
			plot.setRouteType(gc.getInfoTypeForString("NO_ROUTE")) 
			return true

		return false

	def destroyImprovement(self, plot):
		"destroys any improvements"

		if (plot.getImprovementType() != gc.getInfoTypeForString("NO_IMPROVEMENT")):
			plot.setImprovementType(gc.getInfoTypeForString("NO_IMPROVEMENT"))
			return true

		return false

	def setUpEarthquake(self):
		"set up a new earthquake"

		#CvUtil.pyPrint("entered setupearthquake")

		validplot = false
		maxX = CyMapInstance.getGridWidth() - 1
		maxY = CyMapInstance.getGridHeight() - 1
		newX = -1
		newY = -1
		newplot = None
		
		#limit searching for a valid plot to 100 tries to avoid infinite loop
		trylimit = 100
		tries = 0

		#find a random plot in the world that is not ice
		while (validplot == false and tries < trylimit):
			newX = random.randint(0, maxX)
			newY = random.randint(0, maxY)
			newplot = CyMapInstance.plot(newX, newY)
			
			if (newplot.getFeatureType() != gc.getInfoTypeForString("FEATURE_ICE")):
				validplot = true
			else:	
				tries += 1

		#CvUtil.pyPrint("tries was " + str(tries))
		#CvUtil.pyPrint("plot " + str(newplot.getX()) + "," + str(newplot.getY()))
		
		#return true or false depending on whether an earthquake plot was found      
	
		if (validplot == true):
                	self.earthquakePlot = newplot

			self.severity = random.randint(1, EARTHQUAKE_MAXPOWER)
			#self.severity = 100  #DEBUG
        		return true
		else:
			self.earthquakePlot = None
			return false


	

	def turnChecker(self, iTurnNum):
		"this is called every turn.  Checks to see if the earthquake event should occur this turn"
		
		#CvUtil.pyPrint("entered turnchecker")

		#the chance of an earthquake is only determined every EARTHQUAKE_DELAY turns

		if (iTurnNum % EARTHQUAKE_DELAY == 0):

			randnum = random.randint(1, EARTHQUAKE_MAX_CHANCE)

			if (randnum <= EARTHQUAKE_CHANCE_LIMIT):

				setupSuccess = false

        			setupSuccess = self.setUpEarthquake()
        
				if (setupSuccess == true):
					self.checkEra()
							
					self.showScreen()

