## 
# Locust Swarm Event
# Author - David Fidge
#
# Version 1.4
# - added new landmark code
# Version 1.2
# - removed some constants
# Version 1.1
# - removed popup code
# - other minor changes
# Version 1.0
# - original release


from CvPythonExtensions import *
from RandomEventsConstants import *
from CvRandomEventsScreenEnums import *
import CvRandomEventsScreenUtils
import GlobalFunctions
import LandmarkManager


import RandomEventsIO
import RandomEventsScreen

import CvUtil
import random

#import CvScreenEnums

gc = CyGlobalContext()
CyMapInstance = gc.getMap()
GlobalFunctions = GlobalFunctions.GlobalFunctions()
LandmarkManager = LandmarkManager.LandmarkManager()

#the main class

class RandomEventLocustSwarm:
	"Locust Swarm Event Class"
	
	def __init__(self, io):
		#set the io object - should always be done last as a reminder to not use the object yet 
		#as game may have not started
		self.IOinstance = io

	def initialise(self):
		"initialise the mod.  This is called when loading a game or starting a new game"
		if (self.isSwarmInProgress() == true):
			self.displaySwarm()

	def isSwarmInProgress(self):
		"returns true if swarm is in progress, false otherwise"

		x, y = self.IOinstance.getLocustSwarmOccupyingPlot()

		if (x == -1 and y == -1):
			return false
		else:
			return true
		

	def showScreen(self):
		"show the popup screen for a new swarm"

		x, y = self.IOinstance.getLocustSwarmOccupyingPlot()

		#set variables in the popup screen so it can display certain information

		CvRandomEventsScreenUtils.randomEventsScreen.setLocustSwarmScreenVars(CyMapInstance.plot(x, y))

		#put popup in the "screens that need to be shown queue"

		CvRandomEventsScreenUtils.randomEventsScreen.pushScreen(LOCUSTSWARM_QUEUE_ID)

	def updateSwarm(self):
		"update the state of the swarm"

		#init end swarm flag
		endswarmflag = false

		#get x and y value of plot swarm occupying
		x, y = self.IOinstance.getLocustSwarmOccupyingPlot()

		#yum yum!! destroy the farm improvement on the current slot with the locust Swarm

		if (CyMapInstance.plot(x, y).getImprovementType() == gc.getInfoTypeForString("IMPROVEMENT_FARM")):  
			CyMapInstance.plot(x, y).setImprovementType(gc.getInfoTypeForString("NO_IMPROVEMENT"))
			
		#hide the Swarm graphic from the screen

		self.hideSwarm()
		
		#keep on moving in the +x direction until a plot without a city on it is found

		x = GlobalFunctions.getNextX(x, 1)
		if (x == -1):
			#otherwise end the swarm
			endswarmflag = true

				
		if (endswarmflag == false):

			#if swarm wasnt ended by no map wrap do more checks

			if (CyMapInstance.plot(x, y).isCity() == true or CyMapInstance.plot(x, y).isWater() == true or CyMapInstance.plot(x, y).isLake() == true):
				#if we hit a city or water plot then end the swarm
				endswarmflag = true
		
		if (endswarmflag == false):
			#swarm is still active, update the current location of the swarm
			self.IOinstance.setLocustSwarmOccupyingPlot(x,y)

			#display the swarm
			self.displaySwarm()

		else:
			#end the swarm as we hit a city or water plot 
			self.endSwarm()


	def displaySwarm(self):
		"display the swarm landmark"

		x, y = self.IOinstance.getLocustSwarmOccupyingPlot()

		LandmarkManager.addLandmarkText(CyMapInstance.plot(x, y), "Locust Swarm")

		####ALL THE CODE I TRIED TO DISPLAY A 2D GRAPHIC ON THE MAP DIDNT WORK
		####IT MAY BE POSSIBLE TO DRAW ON MAIN INTERFACE SCREEN BUT THIS IS A CLUMSY SOLUTION

		####show the locust Swarm graphic on the screen
		####if(CyMapInstance.plot(x, y)).isRevealed(##teamid goes here)): ##dont need this for landmark
		####if (not CyInterface().isCityScreenUp() and CyInterface().getShowInterface() != InterfaceVisibility.INTERFACE_HIDE_ALL and CyEngine().isGlobeviewUp() == false
		####addPlotGraphicGFC("SwarmGfx"0, 0, CyMapInstance.plot(x, y)
		####if (not CyInterface().isCityScreenUp() and CyInterface().getShowInterface() != InterfaceVisibility.INTERFACE_HIDE_ALL and CyEngine().isGlobeviewUp() == false
		####CvScreensInterface.mainInterface.screen.addGraphic("SwarmGfx", "", "art\locust_mapicon.dds", 0, 0, 0, WidgetTypes.WIDGET_GENERAL, 0, 0)
		####CvScreensInterface.mainInterface.screen.addGraphic("SwarmGfx2", "", "art\locust_mapicon.dds", 10, 10, 1000, WidgetTypes.WIDGET_GENERAL, 0, 0)
		####screen = CyGInterfaceScreen( "MainInterface", CvScreenEnums.MAIN_INTERFACE )
		####screen = CyInterface()
		####for index in range 100:
		####plot = gc.getMap().plot(self.ind, 20)
		####screen.addWorldWidget("SwarmGfxHolder", "MainInterface", -110.0, -110.0, -100.0, 512.0, 512.0, plot.getPoint(), CyCamera().GetCurrentPosition(), true, WidgetTypes.WIDGET_GENERAL, 0, 0)
		####screen.addWorldWidget("SwarmGfxHolder", "MainInterface", CyCamera().GetCurrentPosition().x, CyCamera().GetCurrentPosition().y, CyCamera().GetCurrentPosition().z, 512.0, 512.0, CyCamera().GetCurrentPosition(), CyCamera().GetCurrentPosition(), true, WidgetTypes.WIDGET_GENERAL, 0, 0)
		####void (string szName, string szAttachTo, float fX, float fY, float fZ, float fW, float fH, const NiPoint3 &position, plot.getPoint(), bool renderUnits, WidgetTypes eWidgetType, int iData1, int iData2)
		####screen.addDDS("SwarmGfx", "SwarmGfxHolder", "art\locust_mapicon.dds",1.0, 1.0, 1.0, 512.0, 512.0, WidgetTypes.WIDGET_GENERAL, 0, 0)	
		####screen.addDDSGFC("SwarmGfx2", "art\locust_mapicon.dds",0, 0, 512, 512, WidgetTypes.WIDGET_GENERAL, 0, 0)
		####screen.addDDSGFCAt("SwarmGfx3", "SwarmGfxHolder", "art\locust_mapicon.dds",0,0,512,512, WidgetTypes.WIDGET_GENERAL, 0, 0, true)
		####screen.addDDS("SwarmGfx4", "MainInterface", "art\locust_mapicon.dds",plot.getPoint().x, plot.getPoint().y, plot.getPoint().z, 512.0, 512.0, WidgetTypes.WIDGET_GENERAL, 0, 0)	
		####screen.addDDS("SwarmGfx5", "MainInterface", "art\locust_mapicon.dds",CyCamera().GetCurrentPosition().x+ 10, CyCamera().GetCurrentPosition().y+ 10, CyCamera().GetCurrentPosition().z + 10, 0.0, 0.0, WidgetTypes.WIDGET_GENERAL, 0, 0)	
		####screen.minimapFlashPlot(plot.getX(), plot.getY(), 2, 10)
		####screen.show("SwarmGfxHolder")
		####screen.show("SwarmGfx")
		####screen.show("SwarmGfx3")
		####screen.show("SwarmGfx4")
		####screen.show("SwarmGfx5")
		####screen.hide("SwarmGfx2")


	def hideSwarm(self):
		"hide the swarm landmark"

		x, y = self.IOinstance.getLocustSwarmOccupyingPlot()
		LandmarkManager.removeLandmarkText(CyMapInstance.plot(x, y), "Locust Swarm")
		

	def endSwarm(self):
		"end the locust swarm"

		#write a locust Swarm has disappeared message to the quickmessages
		CyInterface().addImmediateMessage("The swarm of locusts has ended", "")

		#set the current plot to none
		self.IOinstance.setLocustSwarmOccupyingPlot(-1,-1)

	def setUpSwarm(self):
		"set up a new locust swarm"

		#CvUtil.pyPrint("entered setupswarm")

		validplot = false
		maxX = CyMapInstance.getGridWidth() - 1
		maxY = CyMapInstance.getGridHeight() - 1
		newX = -1
		newY = -1
		newplot = None
		
		#limit searching for a valid plot to 1000 tries to avoid infinite loop
		trylimit = 1000
		tries = 0

		#find a random plot in the world that is a land plot and not ice
		while (validplot == false and tries < trylimit):
			newX = random.randint(0, maxX)
			newY = random.randint(0, maxY)
			newplot = CyMapInstance.plot(newX, newY)
			
			if (newplot.isWater() == true or newplot.isLake() == true):
				newplot = CyMapInstance.plot(newX, newY).getNearestLandPlot()

			if (newplot.getTerrainType() != gc.getInfoTypeForString("TERRAIN_SNOW")): 
				validplot = true
			else:	
				tries += 1

		#CvUtil.pyPrint("tries was " + str(tries))
		#CvUtil.pyPrint("plot " + str(newplot.getX()) + "," + str(newplot.getY()))

        	#keep on moving in the -x direction on the world map until we hit an ocean plot
        	#if we get back to the original plot before hitting a land plot return false 
	
		if (validplot == true):
		
			validplot = false

        		searchX = newplot.getX() - 1
        		searchY = newplot.getY()
        
        		searchplot = CyMapInstance.plot(newplot.getX(), newplot.getY())
        
        		while (searchX != newplot.getX() and validplot == false):
        			searchplot = CyMapInstance.plot(searchX, searchY)
        			if (searchplot.isWater() == true or searchplot.isLake() == true):
					validplot = true
        				if (searchX + 1 > maxX):
						#create swarm in the last land plot (plot we were in before)
						#if conditions here account for wrapping
        					searchX = 0
        					self.IOinstance.setLocustSwarmOccupyingPlot(searchX, searchY)
        				else:
        
        					self.IOinstance.setLocustSwarmOccupyingPlot(searchX + 1, searchY)
        			else:
        				searchX -= 1
        				if (searchX == 0 and CyMapInstance.isWrapX() == true):
        					searchX = maxX
					elif (searchX == 0 and CyMapInstance.isWrapX() == false):
						#map doesnt wrap, that's okay, we'll start the swarm at the edge of the map
						#starting from the last valid land slot
						validplot = true
						self.IOinstance.setLocustSwarmOccupyingPlot(searchX+1, searchY)       
		
		#return true or false depending on whether the locust swarm was created or not      
	
		if (validplot == true):
                
        		return true
		else:

			return false


	

	def turnChecker(self, iTurnNum):
		"this is called every turn.  Checks to see if the locust Swarm event should occur this turn"
		
		#CvUtil.pyPrint("entered turnchecker")

		if (self.isSwarmInProgress() == true):
			#CvUtil.pyPrint("updating swarm")

			self.updateSwarm()

		else:

        		#the chance of a Swarm is only determined every LOCUSTSWARM_LOCUST_DELAY turns
        
        		if (iTurnNum % LOCUSTSWARM_LOCUST_DELAY == 0):
        
        			randnum = random.randint(1, LOCUSTSWARM_MAX_CHANCE)

				#CvUtil.pyPrint("random number for swarm is " + str(randnum))
				
				if (randnum <= LOCUSTSWARM_CHANCE_LIMIT):

					setupSuccess = false

                			setupSuccess = self.setUpSwarm()
                
        				if (setupSuccess == true):
						self.displaySwarm()						
						self.showScreen()

