## 
# Olympics Event
# Author - David Fidge
# 
# Version 1.4
# - fixed modern olympics counter from resetting every time each civ discovers Scientific Method
# - camera now centres on city that won olympics
# Version 1.2
# - removed some constants
# Version 1.1
# - removed popup code
# Version 1.0
# - initial release

from CvPythonExtensions import *
from RandomEventsConstants import *
from CvRandomEventsScreenEnums import *
import CvRandomEventsScreenUtils
import GlobalFunctions

import RandomEventsIO
import RandomEventsScreen

import CvUtil
import PyHelpers

import random
import math

#import CvScreenEnums

PyPlayer = PyHelpers.PyPlayer
PyCity = PyHelpers.PyCity

gc = CyGlobalContext()
GlobalFunctions = GlobalFunctions.GlobalFunctions()


#the main class

class RandomEventOlympics:
	"Olympic random event class"
	
	def __init__(self, io):
		"initialise class on creation"

		self.eraDelay = OLYMPICS_ANCIENT_DELAY

		#pointer to olympic city.  
		pOlympicCity = OLYMPICS_NO_OLYMPIC_CITY

		#player ID that owns city
		pOwnerOlympicCity = OLYMPICS_NO_OLYMPIC_CITY

		#era we're in
		self.era = OLYMPICS_ANCIENT_ERA

		#set the io object - should always be done last as a reminder to not use the object yet 
		#as game may have not started
		self.IOinstance = io


	def initialise(self):
		"Initialise the olympics event.  This is called when loading a game or starting a new game"
		
		#now need to see if a civ has researched railroad
		#assume ancient, then loop through each civ and set to modern if a civ has researched railroad

		self.era = OLYMPICS_ANCIENT_ERA
		self.eraDelay = OLYMPICS_ANCIENT_DELAY

		for iPlayerLoop in range(gc.getMAX_PLAYERS()):
			
			if (PyPlayer(iPlayerLoop).hasResearchedTech(gc.getInfoTypeForString("TECH_RAILROAD"))):

				self.changeToModern()	
				break	#break out of loop when done		


	def isModern(self):
		if (self.era == OLYMPICS_MODERN_ERA):
			return true

		return false

	def changeToModern(self):

		"called during initialisation and also to change to modern era halfway through a game"

		self.era = OLYMPICS_MODERN_ERA
		self.eraDelay = OLYMPICS_MODERN_DELAY	
			

	def showScreen(self):
		"show the Olympic Games popup"
	
		#put popup in the "screens that need to be shown queue"

		CvRandomEventsScreenUtils.randomEventsScreen.pushScreen(OLYMPICS_QUEUE_ID)

		#set variables in the popup screen so it can display certain information

		CvRandomEventsScreenUtils.randomEventsScreen.setOlympicsScreenVars(self.era, self.IOinstance.getOlympicCounter(), self.pOlympicCity.getName())

		#centre the camera on the event
		if (CENTRE_CAMERA_ON_EVENT == true):

			camera = CyCamera()
			camera.JustLookAtPlot(self.pOlympicCity.plot())

	def ApplyOlympicBonus(self):
		"gives a bonus to the owner of the city that hosts the olympic games"

		#add the amount of commerce the city gets to your amount of gold
		currentgold = gc.getPlayer(self.pOwnerOlympicCity).getGold()
		currentgold += self.pOlympicCity.calculateGoldRate()
		gc.getPlayer(self.pOwnerOlympicCity).setGold(currentgold)

		#increase the population of the city by 1
		self.pOlympicCity.changePopulation(OLYMPICS_POPULATION_BONUS)	

		#increase the culture of the city by 10%
		self.pOlympicCity.changeCulture(int(self.pOlympicCity.getCulture() * OLYMPICS_CULTURE_BONUS))

		#would have liked to set a we love the king day but no way to do it 
		#self.pOlympicCity.isWeLoveTheKingDay(true)		

	def CalculateOlympicCity(self):
		"calculate the eligible cities to host the olympics"
		
		cityList = []
		playerList = []
		listsize = 0

		# get contenders for the city by cycling through each player's city list and adding eligible cities 
		# and corresponding player to the two lists

		for iPlayerLoop in range(gc.getMAX_PLAYERS()):
			
			apCityList = PyPlayer(iPlayerLoop).getCityList()
			
			for pCity in apCityList:

				#tempprint = "the city is working on " + str(pCity.getProductionBuilding())
				#print tempprint

				#a city is only eligible for olympics if it is greater than size POPULATION_LIMIT
				#and it has a colosseum

				if (pCity.getPopulation() >= OLYMPICS_POPULATION_LIMIT and pCity.hasBuilding(gc.getInfoTypeForString("BUILDING_COLOSSEUM"))):  
						
					cityList.append(pCity)
					playerList.append(iPlayerLoop)
					listsize = listsize + 1

		#pick a city at random from the list

		if (listsize > 0):

			randnum = random.randint(0, listsize-1)
			#print cityList
			#print randnum
			self.pOlympicCity = cityList[randnum]
			self.pOwnerOlympicCity = playerList[randnum]
		
		else:
			
			self.pOlympicCity = OLYMPICS_NO_OLYMPIC_CITY
			self.pOwnerOlympicCity = OLYMPICS_NO_OLYMPIC_CITY	


	def checkEra(self, argsList):
		"when a civ invents a new technology this function is called.  If tech that was just invented was railroad then change the era to modern"

		iTechType, iTeam, iPlayer, bAnnounce = argsList
		
		if (not self.isModern() and iTechType == gc.getInfoTypeForString("TECH_RAILROAD")):
			#change counter to 1 for the new modern olympics
			self.IOinstance.setOlympicCounter(1)
			self.changeToModern()

	def turnChecker(self, iTurnNum):

		"this is called every turn.  Checks to see if the olympic event should occur this turn"

		"this is called every turn.  Checks to see if the olympic event should occur this turn"
		
		#olympics are held every [self.eraDelay] turns
		#percent symbol is modulus

		if (gc.getGame().getElapsedGameTurns() % self.eraDelay == 0):

			#there will be olympics this turn - if there was an eligible city then apply the bonus to the
			#player and show the screen, if not then dont do anything
        		
			self.CalculateOlympicCity()
        
        		if (self.pOlympicCity != OLYMPICS_NO_OLYMPIC_CITY):
        
        			self.ApplyOlympicBonus()
        
				self.showScreen()

				self.IOinstance.setOlympicCounter(self.IOinstance.getOlympicCounter() + 1)
