## 
# Volcano Event
# 
#
# Author - Andrew Coates
#
# Modified by David Fidge 
#
# Version 1.4
# - added new landmark code
# Version 1.2
# - initial release
import RandomEventsIO
from RandomEventsConstants import *

import CvScreenEnums
import CvUtil
import random
import GlobalFunctions
import LandmarkManager
import math
from CvPythonExtensions import *


#	IMPORTANT INFORMATION
#	
#	All widget names MUST be unique when creating screens.  If you create
#	a widget named 'Hello', and then try to create another named 'Hello', it
#	will modify the first hello.
#
#	Also, when attaching widgets, 'Background' is a reserve word meant for
#	the background widget.  Do NOT use 'Background' to name any widget, but
#	when attaching to the background, please use the 'Background' keyword.

gc = CyGlobalContext()

CyGameInstance = gc.getGame()
CyMapInstance = gc.getMap()
GlobalFunctions = GlobalFunctions.GlobalFunctions()
LandmarkManager = LandmarkManager.LandmarkManager()

#the main class

class RandomEventVolcano:
	
	#class initialisation (called on creation of the class instance within CvScreensInterface)
	
	def __init__(self, io):
		#set the io object - should always be done last as a reminder to not use the object yet 
		#as game may have not started
		self.IOinstance = io

		self.peakList = []

	#initialise the mod.  This is called when loading a game or starting a new game

	def initialise(self):

		maxX = CyMapInstance.getGridWidth() - 1
		maxY = CyMapInstance.getGridHeight() - 1

		#randomly create volcanos for start of game
		numVols = random.randint(int((math.sqrt(maxX * maxY) / VOLCANO_DENSITY)/2), int(math.sqrt(maxX * maxY) / VOLCANO_DENSITY)) + 1
		#numVols = int((math.sqrt(maxX * maxY) / VOLCANO_DENSITY)	#DEBUG
		#CvUtil.pyPrint("sqrt div 2 = " + str(int((math.sqrt(maxX * maxY) / VOLCANO_DENSITY)/2)) + " sqrt = " + str(int(math.sqrt(maxX * maxY) / VOLCANO_DENSITY)) + " numvols = " + str(numVols) + " maxy = " + str(maxY))

		self.buildPeakList()
		
		if (self.IOinstance.getNumVolcanos() == -1):
			self.IOinstance.setNumVolcanos(0)

			#createNewVolcano will now always create a volcano if there is a peak available
			n = 0
			while (n < numVols):
				self.createNewVolcano(False)
				n = n + 1

	def buildPeakList(self):
		"builds a list of all peaks on the map"

		#clear the existing peakList
		self.peakList = []

		maxX = CyMapInstance.getGridWidth() 
		maxY = CyMapInstance.getGridHeight()

		#loop through all plots on the map and add to list if it is a peak
		for x in range(maxX):
			for y in range(maxY):
				newplot = CyMapInstance.plot(x, y)
        			if (newplot.isPeak() == true):

					#add the peak and a 'volcano is here' flag

					if (newplot.getBonusType(CyGame().getActivePlayer()) == gc.getInfoTypeForString("BONUS_VOLCANO")):
						#volcano exists here already, set to true
						self.peakList.append((newplot, true))
						
					else:
						#no volcano exists, set to false
		    				self.peakList.append((newplot, false))
        
		

	def createNewVolcano(self, bShowMsg):
		"create a new random volcano"

		validplot = false

		newplot = None
		peak = None
		existsflag = false
		randnum = 0
		
		#find a peak from the peaklist that is not yet a volcano

		indexlist = []

		for x in range(len(self.peakList)):
			peak, existsflag = self.peakList[x]
			if (existsflag == false):
				indexlist.append(x)

		#CvUtil.pyPrint("valid peak index list: " + str(indexlist))
		#CvUtil.pyPrint("peak List: " + str(self.peakList))

		if (len(indexlist) == 0):
			#all mountains are already volcanos, set no valid plot
			validplot = false

		else:
			#get a random value from the index list, this will be the index into peakList to use for volcano
			randnum = random.randint(0, len(indexlist) - 1)
			newplot, existsflag = self.peakList[indexlist[randnum]]			

			#set volcano exists flag in the peak list
			self.peakList[indexlist[randnum]] = (newplot, true)
			validplot = true

        	if (validplot == true):
			volNum = self.IOinstance.getNumVolcanos() + 1

			volMsg = "A mountain has started rumbling."

			#set the name
			if (volNum <= len(VOLCANO_NAMES)):
				volName = VOLCANO_NAMES[volNum - 1]
			else:
				#use generic name if no more names left (should rarely happen)
				volName = "Volcano " + str(volNum)

			volName = "Volcano: " + volName

			LandmarkManager.addLandmarkText(newplot, volName)

			volMsg = "A mountain has started rumbling, and has been named \"" + volName + "\" by locals."

			
			self.IOinstance.setNumVolcanos(volNum)

			#set the bonus type to our custom volcano bonus number so that it is saved out in the game and is restored on load
			newplot.setBonusType(gc.getInfoTypeForString("BONUS_VOLCANO"))

			#only show message if the mountain is visible to the human (landmark will always be there in any case if human reveals later)
			#if (bShowMsg == true and newplot.isVisibleToWatchingHuman() == True):
			CyInterface().addMessage(CyGame().getActivePlayer(), True, 20, volMsg, "AS2D_VOLCANO_CREATED", 2, "Art/Interface/Buttons/BaseTerrain/Peak.dds", ColorTypes(7), newplot.getX(), newplot.getY(), True, True) 
			
			return true
		else:
			return false
	

	def boomVolcano(self):
		"do volcano eruption"

		validplot = false
		maxX = CyMapInstance.getGridWidth() - 1
		maxY = CyMapInstance.getGridHeight() - 1
		newX = -1
		newY = -1
		newplot = None
		existsflag = false
		randnum = 0	

		#find a peak from the peaklist that is a volcano

		indexlist = []

		for x in range(len(self.peakList)):
			peak, existsflag = self.peakList[x]
			if (existsflag == true):
				indexlist.append(x)

		if (len(indexlist) == 0):
			#all mountains are not volcanos, set no valid plot
			validplot = false

		else:
			#at least 1 volcano, choose one at random for boom
			validplot = true

			randnum = random.randint(0, len(indexlist) - 1)
			newplot, existsflag = self.peakList[indexlist[randnum]]

		if (validplot == true):

			#for all plots surrounding this plot do explodePlot function

			for x in range (-1, 2):
				for y in range (-1, 2):
					if (not (x == 0 and y == 0)):

						tempx = GlobalFunctions.getNextX(newplot.getX(), x)
						tempy = GlobalFunctions.getNextY(newplot.getY(), y)

						if (tempx != -1 and tempy != -1):
							self.explodePlot(CyMapInstance.plot(tempx, tempy))
											

			#get the landmark and show volcano exploded message

			#CyGame().getActivePlayer()
			volName = LandmarkManager.getLandmarkText(newplot)

			if (volName == None):
				volMsg = "A volcano has erupted."
			else:
				volMsg = volName + " has erupted."

			#if (newplot.isVisibleToWatchingHuman() == True):
			#	CyInterface().addMessage(CyGame().getActivePlayer(), True, 20, volMsg, "AS2D_VOLCANO_BOOM", 2, "Art/Interface/Buttons/BaseTerrain/Peak.dds", ColorTypes(7), newplot.getX(), newplot.getY(), True, True) 
			CyInterface().addMessage(CyGame().getActivePlayer(), True, 20, volMsg, "AS2D_VOLCANO_BOOM", 2, "Art/Interface/Buttons/BaseTerrain/Peak.dds", ColorTypes(7), newplot.getX(), newplot.getY(), True, True) 
				
			return true
		else:
			return false

	def explodePlot(self, pplot):
		"Do volcano damage to pplot"

		if (pplot.isCity() == True):
			ccity = pplot.getPlotCity()

			#reduce population
			pop = ccity.getPopulation()
			if (pop < 2):
				ccity.kill()
			else:
				pop = ccity.setPopulation(pop - (pop/VOLCANO_POP_MODIFIER))

		else:
			#Destroy any improvments
			pplot.setImprovementType(-1)
			numUnits = pplot.getNumUnits()
			while (numUnits > 0):

				#kill the unit, kill goes to barbarians
				pplot.getUnit(numUnits - 1).kill(True, gc.getBARBARIAN_PLAYER())
				numUnits = numUnits - 1
	

	def turnChecker(self, iTurnNum):
		"this is called every turn.  Checks to see if the locust Swarm event should occur this turn"
		
		#CvUtil.pyPrint("entered turnchecker")

       		#the chance of a volcano is only determined every VOLCANO_CREATE_DELAY turns
      
       		if (iTurnNum % VOLCANO_CREATE_DELAY == 0):
        
       			randnum = random.randint(1, VOLCANO_CREATE_MAX_CHANCE)
				
			if (randnum <= VOLCANO_CREATE_CHANCE_LIMIT):
				CvUtil.pyPrint("entered createNewVolcano")
				self.createNewVolcano(True)
				
       		if (iTurnNum % VOLCANO_BOOM_DELAY == 0):
        
       			randnum = random.randint(1, VOLCANO_BOOM_MAX_CHANCE)
			
			if (randnum <= VOLCANO_BOOM_CHANCE_LIMIT):
				CvUtil.pyPrint("entered boomVolcano")
				self.boomVolcano()
				
						
