## 
# Random Events class
# Author - David Fidge
# 
# Version 1.2
# - added volcanos
# Version 1.1
# - added earthquakes
# - support for new IO class functionality
# Version 1.0
# - initial release


from CvPythonExtensions import *
#import CvPythonExtensions
#import pydoc
from RandomEventsConstants import *

import RandomEventOlympics
import RandomEventLocustSwarm
import RandomEventEarthquake
import RandomEventVolcano
import RandomEventDrought
import RandomEventFertile
import RandomEventsIO

import RandomEventsScreen
import CvRandomEventsScreenUtils

import CvUtil

class RandomEvents:
	"the random events class.  Game Event notifications are sent here from CvRandomEventsEventManager and this class handles them"

	def __init__(self):

		#pydoc.writedoc(CvPythonExtensions)
		"do stuff on instantiation of the IO object"

		#create IO instance
		self.randomEventsIO = RandomEventsIO.RandomEventsIO()

		#create a new olympics random event
		if (ENABLE_OLYMPICS == true):
			self.randomEventOlympics = RandomEventOlympics.RandomEventOlympics(self.randomEventsIO)

		#create locust swarm random event
		if (ENABLE_LOCUSTSWARM == true):
			self.randomEventLocustSwarm = RandomEventLocustSwarm.RandomEventLocustSwarm(self.randomEventsIO)

		#create earthquake random event
		if (ENABLE_EARTHQUAKE == true):
			self.randomEventEarthquake = RandomEventEarthquake.RandomEventEarthquake(self.randomEventsIO)

		#create volcano random event
		if (ENABLE_VOLCANO == true):
			self.randomEventVolcano = RandomEventVolcano.RandomEventVolcano(self.randomEventsIO)

		#create drought random event
		if (ENABLE_DROUGHT == true):
			self.randomEventDrought = RandomEventDrought.RandomEventDrought(self.randomEventsIO)

		#create fertile random event
		if (ENABLE_FERTILE == true):
			self.randomEventFertile = RandomEventFertile.RandomEventFertile(self.randomEventsIO)


	def handleOnGameStart(self, argsList):
		"Do setup tasks for NEW GAME ONLY"

		#set up the script data variables.  Once set up, these will automatically be saved and loaded into the save game files	
		#note - setupScriptData must always be called first - the variables are set to their initial value for a new game

		#CvUtil.pyPrint("Doing OnGameStart stuff")
	
		self.randomEventsIO.setupScriptDataNewGame()

		if (ENABLE_OLYMPICS == true):
			self.randomEventOlympics.initialise()

		if (ENABLE_LOCUSTSWARM == true):
			self.randomEventLocustSwarm.initialise()

		if (ENABLE_EARTHQUAKE == true):
			self.randomEventEarthquake.initialise()
	
		if (ENABLE_VOLCANO == true):
			self.randomEventVolcano.initialise()

		if (ENABLE_DROUGHT == true):
			self.randomEventDrought.initialise()

		if (ENABLE_FERTILE == true):
			self.randomEventFertile.initialise()


	def handleOnLoadGame(self, argsList):
		"do setup tasks for loading a game (note script data loaded automatically)"

		#CvUtil.pyPrint("Doing OnGameLoad stuff")

		#check the script data and add in any additional entries if loading from an older version of the mod
		#since this function doesnt seem to help in making older versions work it is not called
		#self.randomEventsIO.setupScriptDataLoadGame()

		#initialise events

		if (ENABLE_OLYMPICS == true):
			self.randomEventOlympics.initialise()

		if (ENABLE_LOCUSTSWARM == true):
			self.randomEventLocustSwarm.initialise()

		if (ENABLE_EARTHQUAKE == true):
			self.randomEventEarthquake.initialise()

		if (ENABLE_VOLCANO == true):
			self.randomEventVolcano.initialise()

		if (ENABLE_DROUGHT == true):
			self.randomEventDrought.initialise()

		if (ENABLE_FERTILE == true):
			self.randomEventFertile.initialise()


	def handleOnBeginGameTurn(self, iGameTurn):
		"Do stuff for start of turn.  This is where random events will be called"

		##With my old code:  CIV 4 popup handling seems to be bugged, even though popup states are set as "QUEUED", 
		##the locust popup will come up over the top of the olympic popup
		##so instead there is now only 1 big screen and for each event
		##widgets are removed from the old screen and added in the new screen
		##this is all handled by the RandomEventsScreen class, called on showScreen

		#update state of all random events.  

		if (ENABLE_OLYMPICS == true):
			self.randomEventOlympics.turnChecker(iGameTurn)

        	if (ENABLE_LOCUSTSWARM == true):
	       		self.randomEventLocustSwarm.turnChecker(iGameTurn)

		if (ENABLE_EARTHQUAKE == true):
			self.randomEventEarthquake.turnChecker(iGameTurn)

		if (ENABLE_VOLCANO == true):
			self.randomEventVolcano.turnChecker(iGameTurn)

		if (ENABLE_DROUGHT == true):
			self.randomEventDrought.turnChecker(iGameTurn)

		if (ENABLE_FERTILE == true):
			self.randomEventFertile.turnChecker(iGameTurn)

		#show all popups

		CvRandomEventsScreenUtils.randomEventsScreen.showScreen()

		CvUtil.pyPrint("number of signs = " + str(CyEngine().getNumSigns()))
		


	def handleOnTechAcquired(self, argsList):
		"do stuff when a civ discovers a tech"

		if (ENABLE_OLYMPICS == true):
			self.randomEventOlympics.checkEra(argsList)
