## 
# Olympics Event - Popup Screen
# Author - David Fidge
# 
# Version 1.0
#


from CvPythonExtensions import *
from RandomEventsConstants import *
from CvRandomEventsScreenEnums import *
import CvScreenEnums

import CvRandomEventsScreenUtils
import PyHelpers
import CvUtil

import random


#	IMPORTANT INFORMATION
#	
#	All widget names MUST be unique when creating screens.  If you create
#	a widget named 'Hello', and then try to create another named 'Hello', it
#	will modify the first hello.
#
#	Also, when attaching widgets, 'Background' is a reserve word meant for
#	the background widget.  Do NOT use 'Background' to name any widget, but
#	when attaching to the background, please use the 'Background' keyword.

gc = CyGlobalContext()
#localText = CyTranslator()

#stuff for roman numerals conversion
coding = zip(
    [1000,900,500,400,100,90,50,40,10,9,5,4,1],
    ["M","CM","D","CD","C","XC","L","XL","X","IX","V","IV","I"]
)

#the main class

class RandomEventsScreen:
	"Big screen that shows all popups every turn"
	
	#class initialisation (called on creation of the class instance within CvScreensInterface)
	
	def __init__(self, screenId):


		self.screenQueue = []   #queue for showing screens
		self.screenId = screenId
		self.screen = None
		self.WIDGET_ID = "RandomEventsScreenWidget"   #used to set widget ID strings so that widgets can be easily created and deleted
		self.SCREEN_NAME = "RandomEventsScreen"
		self.nWidgetCount = 0    #used to keep a count of all widgets created, so they can be deleted again after

		#formatting constants
		self.xcentre = 0
		self.ycentre = 0
		self.WIDTH = 500
		self.HEIGHT = 500

		###############olympics vars
		self.era = 0
		self.counter = 0
		self.cityname = ""
		self.damagedone = false

		###############locust swarm vars
		self.swarmPlot = None
	
		###############earthquake vars
		self.earthquakePlot = None
		self.severity = 0
		self.eqera = 0

	def showScreen(self):
		"shows the popup"

		#only show the popup if there are popups queued to be shown
		
		if (len(self.screenQueue) != 0):
        
        		if ( CyGame().isPitbossHost() ):
        			return
        
        		# Create screen
        
        		self.screen = self.getScreen()
        
        		#show the screen on the monitor
        
        		self.screen.showScreen(PopupStates.POPUPSTATE_QUEUED, False)
        
			#make window invisible except for other drawn widgets

        		self.screen.showWindowBackground( False )

			#set popup dimensions - make it cover the whole screen so user cant click elsewhere
        		
        		self.screen.setDimensions(0, 0, self.screen.getXResolution(), self.screen.getYResolution())

			#set the screen resolution values to get the centre of the screen to show widgets properly formatted

			self.xcentre = self.screen.getXResolution() / 2
			self.ycentre = self.screen.getYResolution() / 2
        		
        		# create the main panel. I believe PanelStyles.PANEL_STYLE_MAIN gives the dialog box the rounded blue background
        
        		self.screen.addPanel( "RandomEventsScreenPanel", "", "", true, true, self.xcentre - (self.WIDTH/2) , self.ycentre - (self.HEIGHT/2), self.WIDTH , self.HEIGHT, PanelStyles.PANEL_STYLE_MAIN )
        
        		#create the OK button
        		
        		self.screen.setButtonGFC("Exit", "Okay", "", self.xcentre - (self.WIDTH/2) + 100,self.ycentre + (self.HEIGHT/2) - 50, self.WIDTH - 200, 40, WidgetTypes.WIDGET_GENERAL, -1, -1, ButtonStyles.BUTTON_STYLE_STANDARD )

			#show the first screen in the queue			

			self.showNextScreen()

 
	def handleInput( self, inputClass ):
		"WHEN A SCREEN IS UP, ALL SCREEN EVENTS ARE SENT TO ITS HANDLE INPUT FUNCTION. It continues to send all events to the screen until the screen hides itself"

		#CvUtil.pyPrint("notify code is:  " + str(inputClass.getNotifyCode()))
		#CvUtil.pyPrint("uiflags are:  " + str(inputClass.getFlags()))
		#CvUtil.pyPrint("getData are:  " + str(inputClass.getData()))
		#CvUtil.pyPrint("iItemID are:  " + str(inputClass.getID()))
		#CvUtil.pyPrint("getPythonFile are:  " + str(inputClass.getPythonFile()))
		#CvUtil.pyPrint("getFunctionName are:  " + str(inputClass.getFunctionName()))
		#CvUtil.pyPrint("getData1 are:  " + str(inputClass.getData1()))
		#CvUtil.pyPrint("getData2 are:  " + str(inputClass.getData2()))
		#CvUtil.pyPrint("getOption are:  " + str(inputClass.getOption()))
		#CvUtil.pyPrint("Input Class: ")
		#CvUtil.pyPrint(inputClass)

		morescreens = false

		#big if condition means: the user can either click on the ok button or they can press return or numpad enter to close popup
		#note - itemID is 1 for keypress down and 0 for keypress up

		if ( (inputClass.getFunctionName() == "Exit" and inputClass.getNotifyCode() == NotifyCode.NOTIFY_CLICKED) or ((inputClass.getData() == int(InputTypes.KB_RETURN) or inputClass.getData() == int(InputTypes.KB_NUMPADENTER)) and inputClass.getID() == 1 ) ):

			#delete all widgets and show the next queued screen if there is one, otherwise hide the screen

			self.deleteAllWidgets()
			
			morescreens = self.showNextScreen()

			if (morescreens == false):
				self.deleteAllWidgets()	
				self.screen.hideScreen()

				"return 1 to consume the input - this tells the framework that this screen was able to handle the event"
				return 1
			else:
				return 1
		return 0

	def update(self, fDelta):
		"this function is continuously called every few hundred milliseconds, dont need to do anything"
		return

	def getScreen(self):
		"gets the screen instance"

		return CyGInterfaceScreen(self.SCREEN_NAME, self.screenId)
										
	def getNextWidgetName(self):
		"returns a unique ID for a widget in this screen"

		szName = self.WIDGET_ID + str(self.nWidgetCount)
		self.nWidgetCount += 1
		return szName
				
	def deleteAllWidgets(self):
		"deletes all widgets in the screen"

		i = self.nWidgetCount - 1
		while (i >= 0):
			self.nWidgetCount = i
			self.screen.deleteWidget(self.getNextWidgetName())
			i -= 1

		self.nWidgetCount = 0		

										
	def pushScreen(self, screenQueueNum):
		"pushes a screen onto the queue so it will be showed next time showScreen is called"

		self.screenQueue.append(screenQueueNum)

	def popScreen(self):
		"pops a screen off the start of the queue"

		id = self.screenQueue.pop(0)
		return id

	def showNextScreen(self):
		"shows the next screen, returns true for a screen was in the queue to show, false if queue was empty (no more screens left)"

		if (len(self.screenQueue) != 0):
			id = self.popScreen()
			self.updateScreen(id)
			return true
		else:
			return false


	def updateScreen(self, id):
		"updates the screen by drawing all the widgets onto the screen"

		if (id == OLYMPICS_QUEUE_ID):
			self.showOlympicsScreen()
		elif (id == LOCUSTSWARM_QUEUE_ID):
			self.showLocustSwarmScreen()
		elif (id == EARTHQUAKE_QUEUE_ID):
			self.showEarthquakeScreen()

	def setOlympicsScreenVars(self, era, counter, cityname):
		"called externally.  Updates variables that the screen needs for writing custom text onto the screen or for doing things like centering the camera"

		self.era = era
		self.olympiccounter = counter
		self.cityname = cityname
	
	#roman numerals code credits:
	#Raymond Hettinger, 2005/06/15
	#http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/415384

        def decToRoman(self, num):
            if num <= 0 or num >= 4000 or int(num) != num:
                raise ValueError('Input should be an integer between 1 and 3999')
            result = []
            for d, r in coding:
                while num >= d:
                    result.append(r)
                    num -= d
            return ''.join(result)

	def showOlympicsScreen(self):
		"writes olympic screen widgets onto screen"

		#play a sound when popup appears
		CyAudioGame().Play2DSound("AS2D_BUILD_COLOSSEUM")

		#convert olympic number to roman
		counterRoman = self.decToRoman(self.olympiccounter)            
                
		#create title.  Set relavent text depending on era.  Also set correct graphic
		if (self.era == OLYMPICS_ANCIENT_ERA):
			popupart = "Art/interface/screens/randomevents/discus.dds"
			titletext = "<font=4>Ancient Games of the " + counterRoman + " Olympiad</font>"

		else:
			popupart = "Art/interface/screens/randomevents/olympicrings.dds"
			titletext = "<font=4>Modern Games of the " + counterRoman + " Olympiad</font>"

		self.screen.setLabel(self.getNextWidgetName(), "", titletext, CvUtil.FONT_CENTER_JUSTIFY, self.xcentre, self.ycentre - (self.HEIGHT/2) + 10, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		#create body text that says where olympic games were held
		if (self.cityname != ""):

			eventtext = "<font=4>" + "Olympic games have been held in " + self.cityname + "! " + "</font>"

		else:
			#I have since changed it so that the dialog is only shown if there is an eligible city for the games
			#so this should never happen, but the code is here for it just in case

			eventtext = "<font=4>" + "No cities were big enough to host the olympic games" + "</font>"
		self.screen.setLabel(self.getNextWidgetName(), "", eventtext, CvUtil.FONT_CENTER_JUSTIFY,  self.xcentre, self.ycentre - (self.HEIGHT/2) + 60, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)

		#draw the direct draw surface olympic graphic into the dialog box
		self.screen.addDDSGFC(self.getNextWidgetName(), popupart, self.xcentre - 128, self.ycentre - 100, 256, 256, WidgetTypes.WIDGET_GENERAL, -1, -1 )
	
	def setLocustSwarmScreenVars(self, splot):
		"called externally.  Updates variables that the screen needs for writing custom text onto the screen or for doing things like centering the camera"

		self.swarmPlot = splot

	def showLocustSwarmScreen(self):
 		"writes locust swarm screen widgets onto screen"

		#play a sound when popup appears
		CyAudioGame().Play2DSound("AS2D_SWARM")

		#flash on the minimap where the swarm is in the world

		mainscreen = CyGInterfaceScreen( "MainInterface", CvScreenEnums.MAIN_INTERFACE )
		mainscreen.minimapFlashPlot(self.swarmPlot.getX(), self.swarmPlot.getY(), 2, 5)

		#centre the camera on the event
		if (CENTRE_CAMERA_ON_EVENT == true):
			camera = CyCamera()
			camera.JustLookAtPlot(self.swarmPlot)

		#create the title 
		self.screen.setLabel(self.getNextWidgetName(), "", "<font=4>Locust Swarm</font>", CvUtil.FONT_CENTER_JUSTIFY, self.xcentre, self.ycentre - (self.HEIGHT/2) + 10, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		#create the body text label
		eventtext = "<font=4>" + "A locust swarm is sweeping across the land!" + "</font>"  
		self.screen.setLabel(self.getNextWidgetName(), "", eventtext, CvUtil.FONT_CENTER_JUSTIFY,  self.xcentre, self.ycentre - (self.HEIGHT/2) + 60, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)

		#create the description label
		eventtext2 = "<font=4>" + "Farmers fear the destruction of their crops!" + "</font>"  
		self.screen.setLabel(self.getNextWidgetName(), "", eventtext2, CvUtil.FONT_CENTER_JUSTIFY,  self.xcentre, self.ycentre - (self.HEIGHT/2) + 90, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)


		#draw the direct draw surface locust graphic into the dialog box
		popupart = "Art/interface/screens/randomevents/locust_popup.dds"
		self.screen.addDDSGFC(self.getNextWidgetName(), popupart, self.xcentre - 128, self.ycentre - 100, 256, 256, WidgetTypes.WIDGET_GENERAL, -1, -1 )


	def setEarthquakeScreenVars(self, eqplot, severity, eqera, damagedone):
		"called externally.  Updates variables that the screen needs for writing custom text onto the screen or for doing things like centering the camera"

		self.earthquakePlot = eqplot
		self.severity = severity
		self.eqera = eqera
		self.damagedone = damagedone


	def showEarthquakeScreen(self):
		"writes earthquake screen widgets onto screen"

		#play a sound when popup appears
		CyAudioGame().Play2DSound("AS2D_EARTHQUAKE")

		#flash on the minimap where the earthquake was in the world
		mainscreen = CyGInterfaceScreen( "MainInterface", CvScreenEnums.MAIN_INTERFACE )
		mainscreen.minimapFlashPlot(self.earthquakePlot.getX(), self.earthquakePlot.getY(), 2, 5)

		#centre the camera on the event
		if (CENTRE_CAMERA_ON_EVENT == true):

			camera = CyCamera()
			camera.JustLookAtPlot(self.earthquakePlot)

		#create title
		self.screen.setLabel(self.getNextWidgetName(), "", "<font=4>Earthquake!</font>", CvUtil.FONT_CENTER_JUSTIFY, self.xcentre, self.ycentre - (self.HEIGHT/2) + 10, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1 )

		#create body text
		eventtext = ""

		if (self.severity < EARTHQUAKE_SEVERITY1):
			eventtext = "<font=3>" + "The people felt a very slight shudder of the Earth.</font>"
		elif (self.severity < EARTHQUAKE_SEVERITY2):
			eventtext = "<font=3>" + "The people felt the Earth shudder.  "
			if (self.damagedone):
				eventtext += "Minor collateral damage was reported."
			eventtext += "</font>"
			
		elif (self.severity < EARTHQUAKE_SEVERITY3):
			eventtext = "<font=3>" + "The people felt the Earth shake beneath their feet.  "		
			if (self.damagedone):
				eventtext += "People are trapped beneath the rubble and many buildings and roads need major repairs."
			eventtext += "</font>"

		elif (self.severity <= EARTHQUAKE_MAXPOWER):
			eventtext = "<font=3>" + "The people felt the Earth shake violently!  "		
			if (self.damagedone):
				eventtext += "Massive casualties have been reported and the area has been totally devastated!"
			eventtext += "</font>"

		self.screen.addMultilineText(self.getNextWidgetName(),eventtext, self.xcentre - (self.WIDTH/2) + 10, self.ycentre - (self.HEIGHT/2) + 60, self.WIDTH-20, 100, WidgetTypes.WIDGET_GENERAL, 0, 0,  CvUtil.FONT_CENTER_JUSTIFY)
	
		if (self.eqera == EARTHQUAKE_POST_SEISMIC_ERA):

			eventtext2 = "<font=3>Your scientists report the earthquake was " + str(self.severity/10) + "." + str(self.severity % 10) + " on the Richter scale</font>"

			self.screen.setLabel(self.getNextWidgetName(), "", eventtext2, CvUtil.FONT_CENTER_JUSTIFY,  self.xcentre, self.ycentre - (self.HEIGHT/2) + 110, -2.0, FontTypes.GAME_FONT, WidgetTypes.WIDGET_GENERAL, -1, -1)

		#draw the direct draw surface earthquake graphic into the dialog box
		popupart = "Art/interface/screens/randomevents/earthquake_popup.dds"
		self.screen.addDDSGFC(self.getNextWidgetName(), popupart, self.xcentre - 128, self.ycentre - 100, 256, 256, WidgetTypes.WIDGET_GENERAL, -1, -1 )
	

