#
# Sniper Mod
# CvSniperEventManager
# 

from CvPythonExtensions import *

import Popup as PyPopup

import CvUtil

import CvEventManager
import sys
import PyHelpers
import CvMainInterface
import CvConfigParser
import TheLopezUtils
import math
import time
import SniperUtils
import CityUtils
import CvGameInterface
import CvSniperModGameUtils			
			
gc = CyGlobalContext()	

PyPlayer = PyHelpers.PyPlayer
PyGame = PyHelpers.PyGame()
localText = CyTranslator()
sniperUtils = SniperUtils.SniperUtils()
cityUtils = CityUtils.CityUtils()
gameUtils = CvGameInterface.gameUtils()

# Change this to false prevent snipers from shooting at plots with no enemy 
# units.
# Default value is true
g_bAllowSniperWasteShot = true

# Change this to true to disable the popup allowing players to select which
# unit the sniper should target.
# Default value is false
g_bAlwaysTargetStackLeader = false

# Change this to false if snipers should not be allowed to target specialists
# inside cities.
# Default value is true
g_bAllowSniperTargetSpecialists = true

# Change this to false if snipers should not be allowed to target super 
# specialists inside cities.
# Default value is true
g_bAllowSniperTargetSuperSpecialists = true

# Change this to false if snipers should not be allowed to terrorize citizens
# inside cities.
# Default value is true
g_bAllowSniperTerrorizeCitizens = true

# Change this to true if snipers should be allowed to shoot at any targets,
# even if the snipers owner isn't at war with the owner of the intended
# victim.
# Default value is true
g_bAllowTargetNonEnemies = true

# Change this to true if a war should be automatically declared between the 
# owner of a sniper and the owner of the snipers target when a sniper shoots 
# at their target.
# Default value is true
g_bSniperShotCausesWar = true

# Set to true to print out debug messages in the logs
g_bDebug = false

LOG_ONLY = 1
SCREEN_ONLY = 2
LOG_AND_SCREEN = 3

# Change to redirect the debug messages to the logs, screen or both.
g_iDebugMessageTarget = LOG_ONLY


class CvSniperEventManager:
	targetFoo = None
	cityTarget = false
	aiSniper = None
		
	targetTime = 0.0
	objSelectedPlot = None
	bFatalShot = false
	lastSniper = None
	
	EventLButtonDown=1
	
	def __init__(self, eventManager):
		
		eventDefinition = ('UnitSelectionPopupEvent', self.__eventUnitSelectionPopupApply, self.__eventUnitSelectionPopupBegin)

		eventManager.addCustomEventDefinition(CvSniperModGameUtils.UNIT_SELECTOR_POPUP, eventDefinition)
		
		global g_bAllowSniperWasteShot
		global g_bAlwaysTargetStackLeader
		global g_bAllowSniperTargetSpecialists
		global g_bAllowSniperTargetSuperSpecialists
		global g_bAllowSniperTerrorizeCitizens
		global g_bAllowTargetNonEnemies
		global g_bSniperShotCausesWar

		config = CvConfigParser.CvConfigParser("Sniper Mod Config.ini")

		if(config != None):
			g_bAllowSniperWasteShot = config.getboolean("Sniper Mod", "Allow Sniper Waste Shot", True)
			g_bAlwaysTargetStackLeader = config.getboolean("Sniper Mod", "Always Target Stack Leader", false)
			g_bAllowSniperTargetSpecialists = config.getboolean("Sniper Mod", "Allow Sniper Target Specialists", True)
			g_bAllowSniperTargetSuperSpecialists = config.getboolean("Sniper Mod", "Allow Sniper Target Super Specialists", True)
			g_bAllowSniperTerrorizeCitizens = config.getboolean("Sniper Mod", "Allow Sniper Terrorize Citizens", True)
			g_bAllowTargetNonEnemies = config.getboolean("Sniper Mod", "Allow Target Non-Enemies", True)
			g_bSniperShotCausesWar = config.getboolean("Sniper Mod", "Sniper Shot Causes War", True)

		eventManager.addEventHandler("unitMove", self.onUnitMove)
		eventManager.addEventHandler("Update", self.onUpdate)
		eventManager.addEventHandler("mouseEvent", self.onMouseEvent)
		eventManager.addEventHandler("unitSelected", self.onUnitSelected)
		eventManager.addEventHandler("windowActivation", self.onWindowActivation)
		eventManager.addEventHandler("BeginPlayerTurn", self.onBeginPlayerTurn)
		eventManager.addEventHandler("unitCreated", self.onUnitCreated)
		eventManager.addEventHandler("unitLost", self.onUnitLost)
				
	
	def onBeginPlayerTurn(self, argsList):
		'Called at the beginning of a players turn'
		iGameTurn, iPlayer = argsList

		objPlayer = gc.getPlayer(iPlayer)
		
		for i in range(objPlayer.getNumUnits()):
			objUnit = objPlayer.getUnit(i)
			
			if(objUnit.getUnitCombatType() != gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):
				continue
			
			sniperUtils.refreshSniper(objUnit)
			
	
	def onWindowActivation(self, argsList):
		'Called when the game window activates or deactivates'
		bActive = argsList[0]

		objUnit = CyInterface().getHeadSelectedUnit()

		# If the window was activated then highlight the appropriate sniper 
		# plots and trigger the onUpdate method
		if(bActive and objUnit != None):
			sniperUtils.handleSniperPlotHighlighting(CvGameInterface.gameUtils().isTakeShotEnabled())
			self.onUpdate(argsList)
			CyInterface().clearSelectionList()
						
						
	def onMouseEvent(self, argsList):
		'mouse handler - returns 1 if the event was consumed'
		eventType,mx,my,px,py,interfaceConsumed,screens = argsList
		
		print "onMouseEvent"
		print eventType

		
		# handle the mouse event if we got a valid plot location and the left
		# mouse button was pressed.		
		if ( px!=-1 and py!=-1 and eventType == self.EventLButtonDown ):
			objTargetPlot = CyMap().plot(px, py)

			#CyInterface().addImmediateMessage("%s %s" %(gc.getGame().GetWorldBuilderMode(),objTargetPlot.getUnit(0).getExtraFatalShotChance()),"")
			if(CvGameInterface.gameUtils().isTakeShotEnabled()):
				self.handleSniperTakeShot(objTargetPlot)	
					
	
	def onUpdate(self, argsList):
		'Called every frame'
		fDeltaTime = argsList[0]
		
		if(not gc.getGame().GetWorldBuilderMode()):
			sniperUtils.handleSniperPlotHighlighting(CvGameInterface.gameUtils().isTakeShotEnabled())
			
			self.targetTime += fDeltaTime

			if(self.targetFoo != None and self.targetTime > 1.15):

				targetUnitEntity = CyUnitEntity(self.targetFoo)
				subEntityCount = targetUnitEntity.GetSubEntityCount()
				if(targetUnitEntity.GetUnitsCurrentlyAlive() == 1):
					subEntityCount = 1

				nextDamage = (100/subEntityCount)
				if(not self.bFatalShot):
					nextDamage = 0
					
				currentDamage = self.targetFoo.getDamage() + nextDamage
				
				# Debug code - start
				if(g_bDebug):
					self.printDebugMessage("Entity Count: %s Current Damage: %s  Fatal Shot? %s" %(subEntityCount, currentDamage, self.bFatalShot))
				# Debug code - end
				
				objSniper = None
				
				if(self.lastSniper != None):	
					objSniper = self.lastSniper
				if(self.aiSniper != None):
					objSniper = self.aiSniper
					self.aiSniper = None
					
				sniperUtils.giveExperience(objSniper, self.bFatalShot)
							
				self.targetFoo.setDamage(currentDamage,0)			
				self.targetFoo = None
				self.targetTime = 0.0
				self.bFatalShot = false
				
			elif(self.cityTarget and self.targetTime > 1.15):
				if(self.bFatalShot):
					CyInterface().playGeneralSound("AS3D_UN_DEATH_VOX")
				else:
					CyInterface().playGeneralSound("AS3D_UN_HIT_VOX")
				
				self.bFatalShot = false
				self.targetTime = 0.0
				self.cityTarget = false
		return 0
		
		
	def onUnitCreated(self, argsList):
		'Unit Completed'
		objUnit = argsList[0]

		# If the unit is a sniper then set the line of sight promotion		
		if(objUnit.getUnitCombatType() == gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):
			sniperUtils.setLineOfSight(objUnit)
			sniperUtils.setupSniperData(objUnit)

		
	# Adds or removes the line of sight promotion from sniper units depending
	# on what type of terrain the sniper is on		
	def onUnitMove(self, argsList):
		'unit move'

		objUnit = argsList[1]
		
		# Get the plot the sniper is on
		objPlot = objUnit.plot()
		
		# If the unit is a sniper then set the line of sight promotion		
		if(objUnit.getUnitCombatType() == gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):
			sniperUtils.setLineOfSight(objUnit)
			sniperUtils.updateSniperData(objUnit)
			sniperUtils.fixOldPlot(objUnit)
			
		sniperUtils.scanForSniper(objUnit)

		if(objUnit.getUnitCombatType() != gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):
			objSniper = sniperUtils.getClosestHarassingSniper(objUnit)
			
			if(objSniper != None and sniperUtils.canSniperHarassUnits(objSniper)):
				# Debug code - start
				if(g_bDebug):
					self.printDebugMessage("Sniper harassing: "+ gc.getPlayer(objUnit.getOwner()).getName() + "'s " + objUnit.getName() + " unit")
				# Debug code - end

				sniperUtils.setUnitFacingDirection(objSniper, objUnit.plot())

				self.fireShot(objSniper, objUnit)
				

	def onUnitLost(self, argsList):
		'Unit Lost'
		objUnit = argsList[0]

		# If the unit is a sniper then set the line of sight promotion		
		if(objUnit.getUnitCombatType() == gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):

			sniperUtils.fixOldPlot(objUnit)
			sniperUtils.deleteSniperData(objUnit)
	

	def onUnitSelected(self, argsList):
		'Unit Selected'
		unit = argsList[0]

		gameUtils.setCurrentSniper()
		
		
	def __eventUnitSelectionPopupBegin(self):
		pass
		   
		   
	def __eventUnitSelectionPopupApply(self, playerID, netUserData, popupReturn):
		pPlayer = gc.getActivePlayer()
		iPlayer = pPlayer.getID()

		iTarget = popupReturn.getButtonClicked()
		objUnit = CyInterface().getHeadSelectedUnit()
		
		# Get the in-game entity for the unit passed in.
		objUnitEntity = CyUnitEntity(objUnit)
		
		if(iTarget >= 0):
			self.fireShot(objUnit,self.objSelectedPlot.getUnit(iTarget)) 
		elif(self.objSelectedPlot.isCity() and iTarget < 0 and iTarget != -4200 and iTarget != -1):
			if(not sniperUtils.isTargetHit(objUnit,self.objSelectedPlot)):
				sniperUtils.missedShot(objUnit)
				return		

			objCity = self.objSelectedPlot.getPlotCity()
			iTarget = (iTarget*-1)-30

			self.bFatalShot = sniperUtils.isShotFatal(objUnit)
			strAdjective = "hurt"
			
			cityUtils.citizensFleeToCity(objCity)
			
			if(self.bFatalShot and cityUtils.changeCitySpecialistPopulation(objCity, iTarget)):
				strAdjective = "killed"
			else:
				self.bFatalShot = false
				
			objUnitEntity.GetSubEntity(0).PlayAnimationPath(AnimationPathTypes.ANIMATIONPATH_MELEE_STRIKE)	

			objSpecialistInfo = gc.getSpecialistInfo(iTarget)		
			CyInterface().addImmediateMessage("Your sniper successfully %s a %s" %(strAdjective, objSpecialistInfo.getDescription()),"")		

			objCity = self.objSelectedPlot.getPlotCity()

			self.cityTarget = true 
			self.targetTime = 0.0

			sniperUtils.giveExperience(self.lastSniper, self.bFatalShot)
			

		elif(iTarget == -4200):
			if(not sniperUtils.isTargetHit(objUnit,self.objSelectedPlot)):
				sniperUtils.missedShot(objUnit)
				return		
			objUnitEntity.GetSubEntity(0).PlayAnimationPath(AnimationPathTypes.ANIMATIONPATH_MELEE_STRIKE)	
		
			objCity = self.objSelectedPlot.getPlotCity()
			objCity.changeConscriptAngerTimer(1)
			self.bFatalShot = sniperUtils.isShotFatal(objUnit)
			
			if(cityUtils.citizensFleeToCity(objCity)):		
				sniperUtils.giveExperience(self.lastSniper, self.bFatalShot)
				CyInterface().addImmediateMessage("Your sniper has successfully terrorized the population of " + objCity.getName(),"")		
			else:
				CyInterface().addImmediateMessage("The population of " + objCity.getName() + " is already living in terror","")

 			self.cityTarget = true
			self.targetTime = 0.0
					

		self.objSelectedPlot = None
				
		
	# Handles the cases where the sniper shots at a unit. If there are multiple 
	# units in the target plot then the player will be allowed to select one of 
	# the units in the plot.
	def handleSniperTakeShot(self, objTargetPlot):

		# Return immediately if the target plot passed in is invalid
		if(objTargetPlot == None):
			return

		# Return immediately if the target plot in is invalid
		if(objTargetPlot.isNone()):
			return
			
		# Return immediately if the take shot action isn't enabled.
		if(not gameUtils.isTakeShotEnabled()):
			return	
			
		objUnit = CyInterface().getHeadSelectedUnit()
		if(objUnit == None):
			return
			
		self.lastSniper = objUnit
		
		objPlayer = gc.getPlayer(objUnit.getOwner())
		
		# Return immediately if the target plot passed in is invalid
		if(objTargetPlot == None):
			return
		
		# Get the number of targets on the target plot.
		iSniperTargets = sniperUtils.getSniperTargetCount(objUnit, objTargetPlot)
		self.objSelectedPlot = objTargetPlot
			
		# Return immediately if no unit is currently selected
		if(objUnit == None):
			return

		if(not objUnit.canMove()):
			gameUtils.takeShotCancelled()
			return
			
		# Get the in-game entity for the unit passed in.
		objUnitEntity = CyUnitEntity(objUnit)

		# Return immediately if the current unit selected isn't a sniper.
		if(objUnit.getUnitCombatType() != gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):
			return
		
		objSniperPlot = objUnit.plot()
		
		# Return immediately if the target plot is the same plot as the snipers
		if(objSniperPlot.getX() == objTargetPlot.getX() and objSniperPlot.getY() == objTargetPlot.getY()):
			return
		
		# Set the units facing direction
		sniperUtils.setUnitFacingDirection(objUnit, objTargetPlot)
				
		# Get the snipers shooting range
		iSniperShootingRange = objUnit.getExtraVisibilityRange()

		if(iSniperShootingRange <= 0):
			iSniperShootingRange = 1

		# Get the snipers visibility range
		iSniperVisibilityRange = objUnit.visibilityRange()
		
		# If the snipers line of sight is blocked, inform the user and return
		if(sniperUtils.isLineOfSightBlocked(objSniperPlot, objTargetPlot)):
			strMessage = localText.getText("TXT_KEY_LINE_OF_SIGHT_OBSTRUCTED", ())
			CyInterface().addImmediateMessage(strMessage,"") 
			return

		# If the sniper's target is out of range, inform the user and return
		if(TheLopezUtils.getPlotDistance(objSniperPlot, objTargetPlot) > iSniperShootingRange):
			strMessage = localText.getText("TXT_KEY_OUT_OF_RANGE", ())
			CyInterface().addImmediateMessage(strMessage,"") 
			return
		
		# If the sniper has no target then return
		if(iSniperTargets == 0):
			
			# Before returning, if the mod is configured to allow the wasting 
			# of shots then waste a shot
			if(g_bAllowSniperWasteShot):
				sniperUtils.wasteShot(objUnit)
			return
								
		if(not g_bAlwaysTargetStackLeader and iSniperTargets > 1):

			if(not objTargetPlot.isFriendlyCity(objPlayer.getTeam())):
				self.unitSelectionPopup(objTargetPlot)
			return
		
		objTargetUnit = objTargetPlot.getUnit(0)

		self.fireShot(objUnit, objTargetUnit)


	# Fires the shot from the unit passed in to the target unit		
	def fireShot(self, objUnit, objTargetUnit):

		# Return immediately if the unit passed in is invalid
		if(objUnit == None):
			return

		# Return immediately if the unit in is invalid
		if(objUnit.isNone()):
			return

		# Return immediately if the unit passed in isn't a sniper.
		if(objUnit.getUnitCombatType() != gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):
			return
					
		# Return immediately if the target unit passed in is invalid
		if(objTargetUnit == None):
			return

		# Return immediately if the target unit in is invalid
		if(objTargetUnit.isNone()):
			return			

		# Get the team for the sniper
		objUnitTeam = gc.getTeam(objUnit.getTeam())
		
		# If the mod is configured to not allow targetting of non-enemies and 
		# the owner of the sniper isn't at war with the owner of the target 
		# then display the invalid target message and return
		if(not g_bAllowTargetNonEnemies and not objUnitTeam.isAtWar(objTargetUnit.getTeam())):
			strTargetPlayerName = gc.getPlayer(objTargetUnit.getOwner()).getName()
			strMessage = localText.getText("TXT_KEY_INVALID_TARGET", (strTargetPlayerName,))
			CyInterface().addImmediateMessage(strMessage,"") 
			return
		
		# If the mod is configured to start a war when a sniper shoots at a 
		# target and their owner isn't at war with the owner of the target then 
		# start a war between them.
		if(g_bSniperShotCausesWar and not objUnitTeam.isAtWar(objTargetUnit.getTeam())):
			objUnitTeam.declareWar(objTargetUnit.getTeam(), false)
						
		# If the target unit is a sniper and they are not visible then return
		if(objTargetUnit.getUnitCombatType() == gc.getInfoTypeForString("UNITCOMBAT_SNIPER") and not objTargetUnit.plot().isInvisibleVisible(objUnit.getTeam(), gc.getInfoTypeForString("INVISIBLE_SNIPER"))):
			return

		# if the target unit isn't hit by the unit then perform the missed shot 
		# and return			
		if(not sniperUtils.isTargetHit(objUnit, objTargetUnit.plot())):
			sniperUtils.missedShot(objUnit)
			return
						
		# Get the in-game entity for the unit passed in.
		objUnitEntity = CyUnitEntity(objUnit)
		
		# Get the if the shot was fatal or not
		self.bFatalShot = sniperUtils.isShotFatal(objUnit)

		# Return if the target unit belongs to the owner of the unit passed in
		if(objTargetUnit.getOwner() == objUnit.getOwner()):
			return
				
		# Get the target unit entity instance
		targetUnitEntity = CyUnitEntity(objTargetUnit)
			
		# If no target unit entity was return then return
		if(targetUnitEntity == None):
			return
		
		iWho = targetUnitEntity.GetUnitsCurrentlyAlive()-1

		objUnitEntity.GetSubEntity(0).PlayAnimationPath(AnimationPathTypes.ANIMATIONPATH_MELEE_STRIKE)
		if(targetUnitEntity.GetSubEntity(iWho) != None):
			if(self.bFatalShot):
				targetUnitEntity.GetSubEntity(iWho).PlayAnimationPath(AnimationPathTypes.ANIMATIONPATH_MELEE_DIE)
				strMessage = localText.getText("TXT_KEY_TARGET_HIT_KILL", ())
			else:
				targetUnitEntity.GetSubEntity(iWho).PlayAnimationPath(AnimationPathTypes.ANIMATIONPATH_MELEE_HURT)
				strMessage = localText.getText("TXT_KEY_TARGET_HIT", ())
			
			CyInterface().addImmediateMessage(strMessage,"") 

		if(sniperUtils.isSniperHarassingEnabled(objUnit)):			
			self.lastSniper = objUnit
			objTargetUnit.finishMoves()

		self.targetFoo = objTargetUnit
		self.targetTime = 0.0

		if(not sniperUtils.isSniperHarassingEnabled(objUnit)):
			if(objUnit.isHasPromotion(gc.getInfoTypeForString("PROMOTION_SNIPER"))):
				objUnit.setMoves(objUnit.getMoves()+60)
			else:
				objUnit.finishMoves()

			if(not objUnit.canMove()):
				gameUtils.takeShotCancelled()
						
		
	# Fires the shot from the unit passed in to the target unit		
	def AI_fireShot(self, objUnit, objTargetUnit):

		# Return immediately if the unit passed in is invalid
		if(objUnit == None):
			return

		# Return immediately if the unit in is invalid
		if(objUnit.isNone()):
			return

		# Return immediately if the unit passed in isn't a sniper.
		if(objUnit.getUnitCombatType() != gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):
			return
					
		# Return immediately if the target unit passed in is invalid
		if(objTargetUnit == None):
			return

		# Return immediately if the target unit in is invalid
		if(objTargetUnit.isNone()):
			return			
						
		# If the target isn't hit then perform the missed shot action and 
		# return
		if(not sniperUtils.isTargetHit(objUnit, objTargetUnit.plot())):
			sniperUtils.missedShot(objUnit)
			return
			
		sniperUtils.setUnitFacingDirection(objUnit, objTargetUnit.plot())
			
		# Get the in-game entity for the unit passed in.
		objUnitEntity = CyUnitEntity(objUnit)
		
		self.bFatalShot = true#sniperUtils.isShotFatal(objUnit)
			
		targetUnitEntity = CyUnitEntity(objTargetUnit)
			
		if(targetUnitEntity == None):
			return
		
		iWho = targetUnitEntity.GetUnitsCurrentlyAlive()-1

		objUnitEntity.GetSubEntity(0).PlayAnimationPath(AnimationPathTypes.ANIMATIONPATH_MELEE_STRIKE)
		if(targetUnitEntity.GetSubEntity(iWho) != None):
			if(self.bFatalShot):
				targetUnitEntity.GetSubEntity(iWho).PlayAnimationPath(AnimationPathTypes.ANIMATIONPATH_MELEE_DIE)
			else:
				targetUnitEntity.GetSubEntity(iWho).PlayAnimationPath(AnimationPathTypes.ANIMATIONPATH_MELEE_HURT)
								
		self.targetFoo = objTargetUnit
		self.targetTime = 0.0
		self.aiSniper = objUnit

		if(objUnit.isHasPromotion(gc.getInfoTypeForString("PROMOTION_SNIPER"))):
			objUnit.setMoves(objUnit.getMoves()+60)
		else:
			objUnit.finishMoves()
	

	# Launches the unit selection popup
	def unitSelectionPopup(self, objPlot): 
		player = gc.getActivePlayer()
		iPlayer = player.getID()
		
		popup = PyPopup.PyPopup(CvSniperModGameUtils.UNIT_SELECTOR_POPUP, EventContextTypes.EVENTCONTEXT_ALL)
		popup.setHeaderString("Select a target:")
		popup.setSize(250,300)
		for iUnitIndex in range(objPlot.getNumUnits()):
			objUnit = objPlot.getUnit(iUnitIndex)

			# Skip the unit if it is a sniper and they are not visible.
			if(objUnit.getUnitCombatType() == gc.getInfoTypeForString("UNITCOMBAT_SNIPER") and not objUnit.plot().isInvisibleVisible(player.getTeam(), gc.getInfoTypeForString("INVISIBLE_SNIPER"))):
				continue
				
			objUnitInfo = gc.getUnitInfo(objUnit.getUnitType())
			strUnitImage = objUnitInfo.getButton()
			popup.addPythonButton(str(objUnit.getID()), objUnit.getName(),objUnit.getName(), strUnitImage, iUnitIndex, -1, true)
		
		if(objPlot.isCity()):
			objCity = objPlot.getPlotCity()
					
			# Add on of each specialist if the mod is configured to allow 
			# snipers to target specialists.			
			if(g_bAllowSniperTargetSpecialists):
				for i in range(1, gc.getNumSpecialistInfos() ):
					objSpecialistInfo = gc.getSpecialistInfo(i)
					if(objCity.getSpecialistCount(i) > 0):
						strSpecialistImage = objSpecialistInfo.getButton()
						index = (i+30)*-1
						strDescription = objSpecialistInfo.getDescription() + " x" + str(objCity.getSpecialistCount(i))
						popup.addPythonButton("", strDescription,strDescription, strSpecialistImage, index, -1, true)

			if(g_bAllowSniperTargetSuperSpecialists):
				for i in range( gc.getNumSpecialistInfos() ):
					objSpecialistInfo = gc.getSpecialistInfo(i)
					if(objCity.getFreeSpecialistCount(i) > 0):
						strSpecialistImage = objSpecialistInfo.getTexture()
						index = (i+30)*-1
						strDescription = objSpecialistInfo.getDescription() + " x" + str(objCity.getFreeSpecialistCount(i))
						popup.addPythonButton("", strDescription,strDescription, strSpecialistImage, index, -1, true)

			if(g_bAllowSniperTerrorizeCitizens):
				strCitizenImage = "Art/Interface/Population Heads/idlecitizen.dds"
				popup.addPythonButton("", "Terrorize Citizens","Terrorize Citizens", strCitizenImage, -4200, -1, true)


		popup.launch(true)		
		
	
	# Prints the debug message to the logs, screen or both	
	def printDebugMessage(self, strMessage):
	
		if(g_iDebugMessageTarget == LOG_ONLY):
			CvUtil.pyPrint(strMessage)
		elif(g_iDebugMessageTarget == SCREEN_ONLY):
			CyInterface().addImmediateMessage(strMessage,"")
		else:
			CvUtil.pyPrint(strMessage)
			CyInterface().addImmediateMessage(strMessage,"")


