## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Implementaion of miscellaneous game functions
# Sniper Mod
# By: The Lopez

import CvUtil
import CvGameUtils
import PyHelpers
import CvConfigParser
import SniperUtils
import TheLopezUtils
import CvEventInterface

from CvPythonExtensions import *

# globals
gc = CyGlobalContext()
localText = CyTranslator()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
sniperUtils = SniperUtils.SniperUtils()


TAKE_SHOT = gc.getGame().getMapRand().get(500,"Action")+1000
HARASS = TAKE_SHOT+1
CANCEL_HARASS = gc.getInfoTypeForString("COMMAND_WAKE")

UNIT_SELECTOR_POPUP = 4204

# Set to true to print out debug messages in the logs
g_bDebug = false

LOG_ONLY = 1
SCREEN_ONLY = 2
LOG_AND_SCREEN = 3

# Change to redirect the debug messages to the logs, screen or both.
g_iDebugMessageTarget = LOG_ONLY

class CvSniperModGameUtils(CvGameUtils.CvGameUtils):

	bTakeShotEnabled = false
	iCurrentSniperID = 0
	
	# Resets the action button numbers
	def initActionButtonNumbers(self):
		global TAKE_SHOT
		global HARASS
		global CANCEL_HARASS
		
		TAKE_SHOT = gc.getGame().getMapRand().get(500,"Action")+1000
		HARASS = TAKE_SHOT+1
		CANCEL_HARASS = gc.getInfoTypeForString("COMMAND_WAKE")
		
		
	# This method is called when there is some type of action that wasn't handled.
	def cannotHandleAction(self,argsList):
	
		if(CANCEL_HARASS == -1):
			self.initActionButtonNumbers()
		
		pPlot = argsList[0]
		iAction = argsList[1]
		bTestVisible = argsList[2]

		if(iAction == TAKE_SHOT):

			objUnit = CyInterface().getHeadSelectedUnit()
			
			self.handleTakeShotPressed(objUnit)
			

		elif(iAction == HARASS):
			objUnit = CyInterface().getHeadSelectedUnit()
		
			self.handleHarassPressed(objUnit)


		elif(iAction == CANCEL_HARASS and CyInterface().getHeadSelectedUnit().getGroup().getActivityType() == ActivityTypes.ACTIVITY_AWAKE):
					
			objUnit = CyInterface().getHeadSelectedUnit()
		
			if(objUnit == None):
				return 
				
			sniperUtils.disableSniperHarassing(objUnit)


		return False


	def handleTakeShotPressed(self, objUnit):
		if(objUnit == None):
			return 
			
		if(self.isTakeShotEnabled()):
			self.takeShotCancelled()
		else:
			self.onTakeShotPressed()
	

	def handleHarassPressed(self, objUnit):
		if(objUnit == None):
			return 

		if(self.isTakeShotEnabled()):
			self.takeShotCancelled()

		objSelectionGroup = objUnit.getGroup()
		objSelectionGroup.setActivityType(ActivityTypes.ACTIVITY_SLEEP)
		objUnit.NotifyEntity(gc.getInfoTypeForString("MISSION_SLEEP"))
		CyInterface().clearSelectionList()			
		sniperUtils.enableSniperHarassing(objUnit)
	
	
	def AI_unitUpdate(self,argsList):
		objUnit = argsList[0]

		if(objUnit.getUnitCombatType() != gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):
			return False
			
		iRange = objUnit.getExtraVisibilityRange()
		if(iRange <= 0):
			iRange = 1
		
		# Get the valid targets for the sniper		
		sniperTargetList = sniperUtils.getSniperTargets(objUnit)
		eventManager = CvEventInterface.getEventManager().getEventManager("CvSniperEventManager")
		if(len(sniperTargetList) > 0):
			eventManager.AI_fireShot(objUnit,sniperTargetList[gc.getGame().getMapRand().get(len(sniperTargetList),"Sniper")])
			# Debug Code - Start
			if(g_bDebug):
				self.printDebugMessage("found %s targets" %(len(sniperTargetList)))			
			# Debug Code - End

			return true
			
		return false
				
	
	def isTakeShotEnabled(self):
		return self.bTakeShotEnabled
		
		
	def onTakeShotPressed(self):
		self.bTakeShotEnabled = true
		CyInterface().setDirty(InterfaceDirtyBits.SelectionButtons_DIRTY_BIT, True)
		
		
	def takeShotCancelled(self):
		self.bTakeShotEnabled = false
 		CyInterface().setDirty(InterfaceDirtyBits.SelectionButtons_DIRTY_BIT, True)
		
		
	def setCurrentSniper(self):
		objUnit = CyInterface().getHeadSelectedUnit()

		if(objUnit.getUnitCombatType() != gc.getInfoTypeForString("UNITCOMBAT_SNIPER")):
			self.takeShotCancelled()
			
		if(objUnit.getID() != self.iCurrentSniperID):
			self.iCurrentSniperID = objUnit.getID()
			self.takeShotCancelled()
